/*
 * Lachesis, an IRCRPG combat engine - Log management
 * Copyright 2002 M. Dannis
 *
 * This program is free software; please see the file COPYING for details.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "root.h"
#include "logcmd.h"
#include "../log.h"
#include "../auth.h"
#include "../utils.h"
#include "../types.h"
#include "../alloc.h"
#include "../lachesis.h"

extern TF log_active;
extern FILE *log_file;
extern CmdOpts options;

#ifndef RPGSERV
extern Cmd_PMod cmd_modules;

void Cmd_logcmd_Init()
{
  Cmd_Alloc(&Cmd_Log_Root, "log", CMD_AUTH_MASTER);
}
#endif

void Cmd_Log_Root(void *ticket, int auth, const char *str)
{
  if (str==NULL||!*str) {
    if (options.logfile!=NULL&&*options.logfile) {
      if (log_file==NULL) {
	Auth_CmdResult(ticket, auth,
		       Util_Format("Existing or invalid logfile: %s",
				   options.logfile, NULL, NULL));
	return ;
      }
      if (log_active)
	Auth_CmdResult(ticket, auth, 
		       Util_Format("Logging to file %s is \2active\2.",
				   options.logfile, NULL, NULL));
      else
	Auth_CmdResult(ticket, auth,
		       Util_Format("Logging to file %s is inactive.",
				   options.logfile, NULL, NULL));
    } else
      Auth_CmdResult(ticket, auth, "No logfile.");
    return ;
  }
  const char *cmd, *data;
  data = str;
  cmd = Util_Separate(&data, 32);
  if (!cmd) {
    cmd = data;
    data = NULL;
  }
  if (strcasecmp(cmd, "FILE")==0) {
    if (data==NULL||!*data)
      Auth_CmdResult(ticket, auth, "Filename required.");
    else {
      Log_Cleanup();
      if (options.logfile!=NULL)
	free(options.logfile);
      options.logfile = strdup(data);
      Log_Init();
    }
    return ;
  }
  if (strcasecmp(cmd, "ON")==0) {
    if (log_file==NULL)
      Auth_CmdResult(ticket, auth, "No logfile open.");
    else {
      log_active=TRUE;
      Auth_CmdResult(ticket, auth, "Logging \2active\2.");
    }
    return ;
  }
  if (strcasecmp(cmd, "OFF")==0) {
    if (log_file==NULL)
      Auth_CmdResult(ticket, auth, "No logfile open.");
    else {
      log_active=FALSE;
      Auth_CmdResult(ticket, auth, "Logging inactive.");
    }
    return ;
  }
  Auth_CmdResult(ticket, auth, Util_Format("Unrecognized command: %s", cmd,
					   NULL, NULL)
		 );
}
